#include <windows.h>
#include "PictureViewPlugin.h"
#include "crt.hpp"
#include "image.hpp"
#include "formatFactory.hpp"
#include "format.hpp"

char version[10];

BOOL WINAPI DllMain(HANDLE hDllHandle, DWORD dwReason, LPVOID lpreserved )
{
  return TRUE;
}

WORD calculateCheckSum(const BYTE* p)
{
  WORD sum = 0;
  for(size_t i = 0; i < sizeof(HoHdr)-2; ++i) sum += *p++;
  return 257*sum + 105;
}


UINT32 __stdcall pvdInit(void) { return PVD_CURRENT_INTERFACE_VERSION; }

void __stdcall pvdExit(void) {}

void __stdcall pvdPluginInfo(pvdInfoPlugin *pPluginInfo)
{
  wsprintf(version, "0.6.%d", Factory::get()->noFormats());
  pPluginInfo->Priority = 7;
  pPluginInfo->pName = "theX";
  pPluginInfo->pVersion = version;
  pPluginInfo->pComments = "Speccy gfx decoder";
}

BOOL __stdcall pvdFileOpen(const char *pFileName, INT64 lFileSize, const BYTE *pBuf, UINT32 lBuf, pvdInfoImage *pImageInfo, void **ppContext)
{
  BYTE *buf  = 0;
  INT64 size = lFileSize ? lFileSize : lBuf;
  const BYTE*  data = pBuf;

  // it can't be our file: too short or too long...
  if(size < sizeof(HoHdr) || size > 256*256 + sizeof(HoHdr)) return FALSE;

  if(lFileSize != 0)
  {
    wchar_t uname[MAX_PATH];
    if(!MultiByteToWideChar(CP_UTF8, 0, pFileName, -1, uname, MAX_PATH)) return FALSE;

    HANDLE file = CreateFileW(uname, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0);
    if(file == INVALID_HANDLE_VALUE) return FALSE;

    buf = new BYTE[lFileSize];

    DWORD read;
    ReadFile(file, buf, lFileSize, &read, 0);
    CloseHandle(file);

    data = buf;
  }

  HoHdr* hdr  = (HoHdr*)data;

  // if it isn't hobeta file
  if( size < (sizeof(HoHdr)+256*hdr->noSex) || hdr->checkSum != calculateCheckSum(data) )
    hdr = 0;
  else
  {
    data += sizeof(HoHdr);
    size -= sizeof(HoHdr);
  }

  Format* fmt = Factory::get()->detect(hdr, data, (size_t)size);

  if(!fmt)
  {
    if(buf) delete[] buf;
    return FALSE;
  }

  Image* img = fmt->convert(hdr, data, (size_t)size);

  if(buf) delete[] buf;

  if(!img) return FALSE;
  if(img->noPages < 1)
  {
    delete img;
    return FALSE;
  }

  pImageInfo->nPages = img->noPages;
  pImageInfo->Flags = 0;
  if(img->animate) pImageInfo->Flags |= PVD_IIF_ANIMATED;

  pImageInfo->pFormatName = img->format ? img->format : "Unknown";

  pImageInfo->pCompression = NULL;
  pImageInfo->pComments = NULL;
  *ppContext = img;
  return TRUE;
}

BOOL __stdcall pvdPageInfo(void *pContext, UINT32 iPage, pvdInfoPage *pPageInfo)
{
  Image* img = (Image*)pContext;
  Page* pg   = img->getPage(iPage);
  if(!pg) return FALSE;

  pPageInfo->lWidth = pg->w;
  pPageInfo->lHeight = pg->h;
  pPageInfo->nBPP = 24;
  if(img->animate) pPageInfo->lFrameTime = 500;
  return TRUE;
}

BOOL __stdcall pvdPageDecode(void *pContext, UINT32 iPage, pvdInfoDecode *pDecodeInfo, pvdDecodeCallback DecodeCallback, void *pDecodeCallbackContext)
{
  Image* img = (Image*)pContext;
  Page* pg   = img->getPage(iPage);
  if(!pg) return FALSE;

  pDecodeInfo->pImage      = (BYTE*)pg->pix;
  pDecodeInfo->pPalette    = NULL;
  pDecodeInfo->Flags       = PVD_IDF_READONLY;
  pDecodeInfo->nBPP        = 24;
  pDecodeInfo->nColorsUsed = 0;
  pDecodeInfo->lImagePitch = 3*pg->w;
  return TRUE;
}

void __stdcall pvdPageFree(void *pContext, pvdInfoDecode *pDecodeInfo) {}

void __stdcall pvdFileClose(void *pContext)
{
  Image* img = (Image*)pContext;
  delete img;
}


// ==================================================================================
#define GFP_BGR 1
#define GFP_READ 0x0001

typedef struct
{
    unsigned char red[256];
    unsigned char green[256];
    unsigned char blue[256];
} GFP_COLORMAP;


BOOL __stdcall gfpGetPluginInfo(DWORD version, LPSTR label, INT label_max_size, LPSTR extension, INT extension_max_size, INT* support)
{
  if(version != 0x0002) return FALSE;

  pvdInfoPlugin pluginInfo;
  pvdPluginInfo(&pluginInfo);

  char full_name[50];
  wsprintf(full_name, "%s: %s v.%s", pluginInfo.pName, pluginInfo.pComments, pluginInfo.pVersion);

  lstrcpyn(label, full_name, label_max_size);
  lstrcpyn(extension, "!s;$s;!f;$8;scr", extension_max_size);
  *support = GFP_READ;

  return TRUE;
}

void* __stdcall gfpLoadPictureInit(LPCSTR filename)
{
  HANDLE file = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0);
  if(file == INVALID_HANDLE_VALUE) return NULL;

  DWORD size = GetFileSize(file, NULL);
  CloseHandle(file);

  if(size == INVALID_FILE_SIZE) return NULL;

  pvdInfoImage imageInfo;
  void* context;

  // transition ANSI -> MultiByte -> UTF8
  wchar_t mbname[MAX_PATH];
  if(!MultiByteToWideChar(GetACP(), 0, filename, -1, mbname, MAX_PATH)) return NULL;

  char uname[MAX_PATH];
  if(!WideCharToMultiByte(CP_UTF8, 0, mbname, -1, uname, MAX_PATH, NULL, NULL)) return NULL;

  if(!pvdFileOpen(uname, size, NULL, 0,  &imageInfo, &context)) return NULL;

  return context;
}

void __stdcall gfpLoadPictureExit(void* ptr)
{
    delete ptr;
}

BOOL __stdcall gfpLoadPictureGetInfo(void* ptr, INT* pictype, INT* width, INT* height, INT* dpi, INT* bits_per_pixel, INT* bytes_per_line, BOOL* has_colormap, LPSTR label, INT label_max_size)
{
  Image* img = (Image*)ptr;
  Page* pg = img->getPage(0);

  if(!pg) return FALSE;

  *width          = pg->w;
  *height         = pg->h;
  *bits_per_pixel = 24;
  *bytes_per_line = sizeof(BGR)*pg->w;
  *pictype        = GFP_BGR;
  *dpi            = 75;
  *has_colormap   = FALSE;

  if(img->format)
   strncpy(label, img->format, label_max_size);
  return TRUE;
}

BOOL __stdcall gfpLoadPictureGetLine(void* ptr, INT line, unsigned char* buffer)
{
  Image* img = (Image*)ptr;
  Page* pg = img->getPage(0);

  if(!pg) return FALSE;

  CopyMemory(buffer, (BYTE*)(pg->pix)+line*sizeof(BGR)*pg->w, sizeof(BGR)*pg->w );
  return TRUE;
}

BOOL __stdcall gfpLoadPictureGetColormap(void* ptr, GFP_COLORMAP* cmap)
{
    return FALSE;
}
